%%%%try with matlab's toolbox (Alex Libby) extract ROI using first image in
%%%%pile 
%%%% updated on 2/16/2016 to adjust for two types of images 
%% clear workspace
close all
clear all

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%USER INPUT%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%Create file directory here: (where text files are saved)
parDIR = '~/Dropbox/microscope data/18_10_30 atc induction for crosstalk/saveFiles';
%%%%Indicate directory of RAW images here: (location of your raw data in
%%%%folders)
fileDIR = '~/Dropbox/microscope data/18_10_30 atc induction for crosstalk/';
%%%Create directory of image figures here: (where you will want the output
%%%mask and fluorescence images to be)
figDIR = '~/Dropbox/microscope data/18_10_30 atc induction for crosstalk/saveFiles';
imageyes = 1; %%%make imageyes = 1 if you want to save figures (see below) 
cellthresh = 500; %%%500 pixels. Most cells are around 1000
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%for saveing file 
cd([parDIR])
datastr = date;
OUTFILE_F1 = ['Channel_2_data_roi_' datastr '.txt'];
fileID_F1 = fopen(OUTFILE_F1, 'wt');
OUTFILE_F2 = ['Channel_3_data_roi_' datastr '.txt'];
fileID_F2 = fopen(OUTFILE_F2, 'wt');
OUTFILE_F3 = ['Channel_4_data_roi_' datastr '.txt'];
fileID_F3 = fopen(OUTFILE_F3, 'wt');
%%%get files types:
cd([fileDIR])
d = dir;
d(~[d.isdir])= []; %Remove all non directories.
foldernames = setdiff({d.name},{'.','..'});

%%
for f = 1:size(foldernames,2)
    
    %%%get file names
    cd([fileDIR '/' foldernames{f}])
    d = dir('*.tif');
    imagenames = {d.name};
    
    %%%go through all images 
    for file = 1:size(imagenames,2)
        
        FileTif = {}; FileTif = imagenames{file};
        
        %%%%%%%%%%%%%%%%%%Use PHASE image to get ROIs%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        [P,map] = imread(FileTif,'Index',1); %%%P - phase (chan1) image
        [F1,map] = imread(FileTif,'Index',2); %%%F - chan2 image
        [F2,map] = imread(FileTif,'Index',3); %%%F - chan3 image
        [F3,map] = imread(FileTif,'Index',4); %%%F - chan4 image
        
        %%basic threshold
        phase = im2double(P);
        basic_threshold = mean(phase(:)) - 1*std(phase(:));
        basic_map = (phase<=basic_threshold).*phase;

        %%%%edges
        diffthreshrow = abs(diff(basic_map)); %%%row differences
        diffthreshcol = abs(diff(basic_map'))';%%%col differences
        distthresh = 5; %%%this is the number of standard deviations away from the mean
        alldiff_threshold = mean(diffthreshcol(:)) + distthresh*std(diffthreshcol(:));
        diffthreshrow(end+1,:) = 0; %%%padding
        diffthreshcol(:, end+1) = 0; %%%padding
        alldiff_map = (abs(diffthreshrow)>alldiff_threshold) + (abs(diffthreshcol)>alldiff_threshold);
        edgesmap = alldiff_map.*phase;
        final_map = logical((basic_map + edgesmap)~=0); 
      
        %STANDARD%basic_threshold = mean(phase(:)) - 1.75*std(phase(:));
        BW = final_map; %BW = imcomplement(im2bw(P, basic_threshold));
        
        %%%clustering
        CC = bwconncomp(BW);
        [clustersizes, ~] = cellfun(@size,CC.PixelIdxList);
        useclusters = find(clustersizes>cellthresh); %%%set cluster threshold here
        clustersizefinal = clustersizes(useclusters);
        dataType = 'uint8';
        L = zeros(CC.ImageSize,dataType);
        
         %%% second channel
        data1 = []; 
        mask1 = zeros(CC.ImageSize,'uint16');
        for k = 1:size(useclusters,2)
            L(CC.PixelIdxList{useclusters(k)}) = k;
            mask1(CC.PixelIdxList{useclusters(k)}) = F1(CC.PixelIdxList{useclusters(k)}); %%%mask of current ROI
            data1(k,1) = k; %%cell number
            data1(k,2) = clustersizefinal(k); %%cell size
            data1(k,3) = sum(F1(CC.PixelIdxList{useclusters(k)}));%%%sum of fluor values at the current ROI
            data1(k,4) = mean(F1(CC.PixelIdxList{useclusters(k)})); %%% mean of fluor values at the current ROI
        end
        
        %%% third channel
        data2 = [];
        mask2 = zeros(CC.ImageSize,'uint16');
        for k = 1:size(useclusters,2)
            mask2(CC.PixelIdxList{useclusters(k)}) = F2(CC.PixelIdxList{useclusters(k)}); %%%mask of current ROI
            data2(k,1) = k; %%cell number
            data2(k,2) = clustersizefinal(k); %%cell size
            data2(k,3) = sum(F2(CC.PixelIdxList{useclusters(k)}));%%%sum of fluor values at the current ROI
            data2(k,4) = mean(F2(CC.PixelIdxList{useclusters(k)})); %%% mean of fluor values at the current ROI
        end
        
        %%% fourth channel
        data3 = [];
        mask3 = zeros(CC.ImageSize,'uint16');
        for k = 1:size(useclusters,2)
            mask3(CC.PixelIdxList{useclusters(k)}) = F3(CC.PixelIdxList{useclusters(k)}); %%%mask of current ROI
            data3(k,1) = k; %%cell number
            data3(k,2) = clustersizefinal(k); %%cell size
            data3(k,3) = sum(F3(CC.PixelIdxList{useclusters(k)}));%%%sum of fluor values at the current ROI
            data3(k,4) = mean(F3(CC.PixelIdxList{useclusters(k)})); %%% mean of fluor values at the current ROI
        end
        
        %%%mask image second channel
        set(gcf,'visible','off')
        if imageyes == 1
            imshow(mask1, [min(mask1(mask1~=0)), max(mask1(:))])
            colorbar
            title(['Cell ROIs FOLDER: ' foldernames{f} ' FILE: ' FileTif]);
            figname = ['F1_' foldernames{f} ' ' regexprep(FileTif,'.tif','')];
            cd([figDIR])
            saveas(gcf,figname,'png')
            close all
        end
        
        %%%mask image third channel
        set(gcf,'visible','off')
        if imageyes == 1
            imshow(mask2, [min(mask2(mask2~=0)), max(mask2(:))])
            colorbar
            title(['Cell ROIs FOLDER: ' foldernames{f} ' FILE: ' FileTif]);
            figname = ['F2_' foldernames{f} ' ' regexprep(FileTif,'.tif','')];
            cd([figDIR])
            saveas(gcf,figname,'png')
            close all
        end
        
        %%%mask image fourth channel
        set(gcf,'visible','off')
        if imageyes == 1
            imshow(mask3, [min(mask3(mask3~=0)), max(mask3(:))])
            colorbar
            title(['Cell ROIs FOLDER: ' foldernames{f} ' FILE: ' FileTif]);
            figname = ['F3_' foldernames{f} ' ' regexprep(FileTif,'.tif','')];
            cd([figDIR])
            saveas(gcf,figname,'png')
            close all
        end
        
        %%%save figures of where ROIs are placed as a double check (note
        %%%you need to specific imageyes = 1 ... see above) 
        set(gcf,'visible','off')
        if imageyes == 1
            imshow(L, [0, size(useclusters,2)], 'Colormap', jet)
            colorbar
            title(['Cell ROIs FOLDER: ' foldernames{f} ' FILE: ' FileTif]);
            figname = [foldernames{f} '_phasemask_' regexprep(FileTif,'.tif','')];
            cd([figDIR])
            saveas(gcf,figname,'png')
            close all
        end

        %%%%F1 ------------ print final voxel_roi to a text file
        cd([parDIR])
        for r = 1:size(data1,1)
            fprintf(fileID_F1, [' %s,'],  foldernames{f});
            fprintf(fileID_F1, [' %s,'],  FileTif);
            fprintf(fileID_F1, [' %d, %d, %d, %d\n'],  data1(r,:));
        end
        cd([fileDIR '/' foldernames{f}])
        
        %%%%F2 ------------ print final voxel_roi to a text file
        cd([parDIR])
        for r = 1:size(data2,1)
            fprintf(fileID_F2, [' %s,'],  foldernames{f});
            fprintf(fileID_F2, [' %s,'],  FileTif);
            fprintf(fileID_F2, [' %d, %d, %d, %d\n'],  data2(r,:));
        end
        cd([fileDIR '/' foldernames{f}])
        
         %%%%F3 ------------ print final voxel_roi to a text file
        cd([parDIR])
        for r = 1:size(data3,1)
            fprintf(fileID_F3, [' %s,'],  foldernames{f});
            fprintf(fileID_F3, [' %s,'],  FileTif);
            fprintf(fileID_F3, [' %d, %d, %d, %d\n'],  data3(r,:));
        end
        cd([fileDIR '/' foldernames{f}])
        
    end
end

fclose('all');
